% z = lassmap(w,l,g,Z)
% Laplacian assignment problem: out-of-sample mapping
%
% Given the assignments Z (matrix of NxK) obtained by LASS for a training set
% of N items (with a Laplacian weight l), the out-of-sample mapping z(x)
% computes the assignment for a test item x, which is represented by its 1xN
% vector w of similarities with the training items and its 1xK vector g of
% similarities with the objects. The assignments z result from solving the
% following QP over soft assignments Z:
%   min_z { |z-v| } st z'.1=1, z>=0
% which is the Euclidean projection of v on the regular simplex, where v is a
% certain function of w, l, g and Z.
% lassmap.m computes the out-of-sample mapping for multiple test items, so w,
% g and z are in general matrices where each row corresponds to one test item.
%
% In:
%   w: MxN matrix, item-item similarities (with the training set items).
%   l: nonnegative scalar, Laplacian regularisation weight.
%   g: MxK matrix, item-object similarities.
%   Z: NxK matrix, item-object assignments (for the training set items).
% Out:
%   z: MxK matrix, item-object assignments.
%
% Any non-mandatory argument can be given the value [] to force it to take
% its default value.

% Copyright (c) 2013 by Miguel A. Carreira-Perpinan

function z = lassmap(w,l,g,Z)

z = SimplexProj(bsxfun(@rdivide,w*Z+g/(2*l),sum(w,2)));

